@@ -1,6 +1,6 @@
 package App::pherkin;
-BEGIN {
-  $App::pherkin::VERSION = '0.15';
+{
+  $App::pherkin::VERSION = '0.16';
 }
 
 use strict;
@@ -8,13 +8,17 @@ use warnings;
 use FindBin::libs;
 use Getopt::Long;
 
+use Moose;
+has 'tags' => ( is => 'rw', isa => 'ArrayRef', required => 0 );
+has 'tag_scheme' => ( is => 'rw', isa => 'ArrayRef', required => 0 );
+
 =head1 NAME
 
 App::pherkin - Run Cucumber tests from the command line
 
 =head1 VERSION
 
-version 0.15
+version 0.16
 
 =head1 SYNOPSIS
 
@@ -49,12 +53,12 @@ Returns a L<Test::BDD::Cucumber::Model::Result> object for all steps run.
 =cut
 
 sub run {
-    my ( $class, @arguments ) = @_;
+    my ( $self, @arguments ) = @_;
 
-    my ($options, @feature_files) = $class->_process_arguments(@arguments);
+    my ($options, @feature_files) = $self->_process_arguments(@arguments);
 
     my ( $executor, @features ) = Test::BDD::Cucumber::Loader->load(
-        $feature_files[0] || './features/'
+        $feature_files[0] || './features/', $self->tag_scheme
     );
     die "No feature files found" unless @features;
 
@@ -62,25 +66,32 @@ sub run {
     my $harness  = $options->{'harness'}->new();
     $harness->startup();
 
-    $executor->execute( $_, $harness ) for @features;
+    my $tag_spec;
+    if ($self->tag_scheme) {
+        $tag_spec = Test::BDD::Cucumber::Model::TagSpec->new({ tags => $self->tag_scheme });
+    }
+
+    $executor->execute( $_, $harness, $tag_spec ) for @features;
 
     $harness->shutdown();
     return $harness->result;
 }
 
 sub _process_arguments {
-    my ( $class, @args ) = @_;
+    my ( $self, @args ) = @_;
     local @ARGV = @args;
 
     # Allow -Ilib, -bl
     Getopt::Long::Configure('bundling');
 
     my $includes = [];
+    my $tags = [];
     GetOptions(
         'I=s@'   => \$includes,
         'l|lib'  => \(my $add_lib),
         'b|blib' => \(my $add_blib),
         'o|output=s' => \(my $harness),
+        't|tags=s@' => \$tags,
     );
     unshift @$includes, 'lib'                   if $add_lib;
     unshift @$includes, 'blib/lib', 'blib/arch' if $add_blib;
@@ -92,9 +103,47 @@ sub _process_arguments {
 
     lib->import(@$includes) if @$includes;
 
+    # Store our TagSpecScheme
+    $self->tag_scheme( $self->_process_tags( @{$tags} ) );
+
     return ({ harness => $harness }, @ARGV);
 }
 
+sub _process_tags {
+    my ( $self, @tags ) = @_;
+
+    # This is a bit faffy and possibly suboptimal.
+    my $tag_scheme = [];
+    my @ands = ();
+
+    # Iterate over our commandline tag strings.
+    foreach my $tag (@tags) {
+        my @parts = ();
+
+        foreach my $part (split(',', $tag)) {
+            # Trim any @ or ~@ from the front of the tag
+            $part =~ s/^(~?)@//;
+
+            # ~@tag => "NOT tag" => [ not => tag ]
+            if (defined $1 and $1 eq '~') {
+                push @parts, [ not => $part ];
+            } else {
+                push @parts, $part;
+            }
+        }
+
+        # @tag,@cow => "@tag OR @cow" => [ or => tag, cow ]
+        # (It's simpler to always stick an 'or' on the front.)
+        push @ands, [ or => @parts ];
+    }
+    # -t @tag -t @cow => "@tag AND @cow" => [ and => tag, cow ]
+    # (It's simpler to always stick an 'and' on the front.)
+    $tag_scheme = [ and => @ands ];
+
+    return $tag_scheme;
+}
+
+
 =head1 AUTHOR
 
 Peter Sergeant C<pete@clueball.com>
@@ -105,4 +154,4 @@ Copyright 2011, Peter Sergeant; Licensed under the same terms as Perl
 
 =cut
 
-1;
\ No newline at end of file
+1;
