@@ -1,6 +1,6 @@
 package Test::BDD::Cucumber::Harness::TermColor;
-BEGIN {
-  $Test::BDD::Cucumber::Harness::TermColor::VERSION = '0.15';
+{
+  $Test::BDD::Cucumber::Harness::TermColor::VERSION = '0.16';
 }
 
 =head1 NAME
@@ -9,7 +9,7 @@ Test::BDD::Cucumber::Harness::TermColor - Prints colorized text to the screen
 
 =head1 VERSION
 
-version 0.15
+version 0.16
 
 =head1 DESCRIPTION
 
@@ -23,6 +23,26 @@ to the terminal.
 use strict;
 use warnings;
 use Moose;
+
+# Try and make the colors just work on Windows...
+BEGIN {
+    if (
+        # We're apparently on Windows
+        $^O =~ /MSWin32/i &&
+        # We haven't disabled coloured output for Term::ANSIColor
+        ( ! $ENV{'ANSI_COLORS_DISABLED'} ) &&
+        # Here's a flag you can use if you really really need to turn this fall-
+        # back behaviour off
+        (! $ENV{'DISABLE_WIN32_FALLBACK'} )
+    ) {
+        # Try and load
+        eval "require Win32::Console::ANSI";
+        if ( $@ ) {
+            print "# Install Win32::Console::ANSI to display colors properly\n";
+        }
+    }
+}
+
 use Term::ANSIColor;
 use Test::BDD::Cucumber::Util;
 use Test::BDD::Cucumber::Model::Result;
@@ -82,12 +102,34 @@ sub step_done {
     } else {
         $color = 'red';
         $follow_up = [ split(/\n/, $result->{'output'} ) ];
+
+        if ( ! $context->is_hook )
+        {
+            unshift @{ $follow_up },
+                'step defined at '
+                . $context->step->line->document->filename
+                . ' line '
+                .  $context->step->line->number
+                . '.';
+        }
+    }
+
+    my $text;
+
+    if ( $context->is_hook )
+    {
+        $color eq 'red' or return;
+        $text = 'In ' . ucfirst( $context->verb ) . ' Hook';
+    }
+    else
+    {
+        $text = $context->step->verb_original . ' ' . $context->text;
     }
 
     $self->_display({
         indent    => 4,
         color     => $color,
-        text      => $context->step->verb_original . ' ' . $context->text,
+        text      => $text,
         highlight => 'bright_cyan',
         trailing  => 0,
         follow_up => $follow_up,
@@ -99,6 +141,7 @@ sub step_done {
 
 sub _note_step_data {
     my ( $self, $step ) = @_;
+    return unless $step;
     my @step_data = @{ $step->data_as_strings };
     return unless @step_data;
 
@@ -180,4 +223,4 @@ Copyright 2011, Peter Sergeant; Licensed under the same terms as Perl
 
 =cut
 
-1;
\ No newline at end of file
+1;
